<?php
namespace App;
/**
 * Newsletters - Main Class
 * 
 * Newsletter and Email marketing solution for SCHLIX CMS
 * 
 * @copyright SCHLIX Web Inc
 *
 * @license SCHLIX Shared Source
 *
 * @package newsletters
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    https://www.schlix.com
 */

class Newsletters_Subscribers extends \SCHLIX\cmsApplication_List {

    protected $data_directories = array(
      'import' => '/data/private/newsletters/import',
      'attachments' => '/data/private/newsletters/attachments',
    );

    
    /**
     * Logger
     * @var \App\Newsletters_EventLog
     */
    protected $eventlog;
    /**
     * Constructor
     */
    
    protected static $default_subscribers_table = 'scxnl_subscribers';
    
    public function __construct() {
        parent::__construct("Newsletters Subscribers", 'scxnl_subscribers');
        // you can modify these to suit your needs
        $this->disable_frontend_runtime = true;
        $this->has_versioning = true;
        $this->eventLog = new \App\Newsletters_EventLog();
        $this->subscriberLog = new \App\Newsletters_SubscriberLog();
        
    }
    

    //_______________________________________________________________________________________________________________//
    /**
     * Deletes items from table. The parameter $mixed_items_to_delete is a pipe-separated
     * value of items, e.g. i4|i5|i1
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $mixed_items_to_delete
     */
    function delete($mixed_items_to_delete)
    {
        global $SystemDB, $CurrentUser;
        
        $mixed_items_array = explode('|', $mixed_items_to_delete); // e.g: i4|i14
        // Process sub-folders first
        foreach ($mixed_items_array as $mixed_item)
        {
            $item_id = substr($mixed_item, 1);
            $items_to_delete[] = $item_id;
        }
        $items_to_delete_str = implode(",", $items_to_delete);
        if ($items_to_delete) {
            $created_by_id = $CurrentUser->getCurrentUserID();
            foreach ($items_to_delete as $delthis)
            {
                $orphan = $this->getItemByID($delthis);
                if ($orphan)
                {
                    $serialized_obj = @serialize($orphan);
                    if ($serialized_obj) {
                        $SystemDB->query("INSERT INTO scxnl_deleted_subscribers (date_created, created_by_id, content) VALUES (NOW(), :created_by_id, :sobj)",
                                ['created_by_id' => $created_by_id, 'sobj' => $serialized_obj]);
                    }
                }
                
            }
            $items_to_delete_str = implode(",", $items_to_delete);
            $sql1 = "DELETE FROM {$this->table_items} WHERE {$this->field_id} in ({$items_to_delete_str})";
            $SystemDB->query($sql1);
            $this->createSubClassInstance('EventLog')->recordAdminActivity("Deleted subscriber(s) with ID: [{$items_to_delete_str}]",'delete','info');
        }
    }    
     /**
     * Static method to quickly check if a subscriber is still active.
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @return bool
     */
    public static function isSubscriberActive($subscriber_id)
    {
        global $SystemDB;
        
        $table_name = static::$default_subscribers_table;
        $id = (int) $subscriber_id;
        $row = $SystemDB->getQueryResultSingleRow("SELECT status FROM {$table_name} WHERE id = {$id}");
        return ($row && ($row['status'] = 1));
    }
    
    public function getSubsriptionsStatusForAllNewsletters($subscriber_id)
    {
        global $SystemDB;
        
        $subscriber_id = (int) $subscriber_id;
        $table_subs = 'scxnl_subscriptions';
        $table_cats = 'scxnl_categories';
        $sql = "SELECT {$table_subs}.*, title, virtual_filename, status  FROM {$table_subs} RIGHT JOIN {$table_cats} ON {$table_cats}.cid = {$table_subs}.newsletter_id WHERE subscriber_id = {$subscriber_id} ORDER BY newsletter_id";
        return $SystemDB->getQueryResultArray($sql);
        
    }
    /**
     * Returns the subscriber info by field real_email_address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $input_email
     * @return array|boolean
     */
    public function getSubscriberByRealEmailAddress($input_email) {
        global $SystemDB;

        if (empty($input_email))
            return false;
        $input_email = \App\Newsletters::getRealEmailAddress($input_email);
        $email = sanitize_string($input_email);
        $sql = "SELECT * from {$this->table_items} where real_email_address <> '' AND  LOWER(real_email_address) = LOWER({$email})";

        $result = $SystemDB->getQueryResultSingleRow($sql);
        return $result;
    } 
    
    /**
     * Return the total number of subscribers with ID greater than $id
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $id
     * @return int
     */
    public function getTotalSubscriberCountWithIDGreaterThan($id)
    {
        global $SystemDB;
        $id = (int) $id;
        $row = $SystemDB->getQueryResultSingleRow("SELECT count(id) as total_count FROM {$this->table_items} WHERE id > {$id} AND status = 1");
        return $row['total_count'];
    }
    
    private function getImportSubscribeRequestID($subscriber_id)
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT MAX(id) FROM scxnl_rq_subscribe WHERE subscriber_id = :subscriber_id AND status = 1", ['subscriber_id' => $subscriber_id]);
        return $row ? $row['id'] : NULL;
    }
    

    protected function getExistingBlackListEntryIDByEmailAddress($email_address)
    {
        global $SystemDB;
        
        $row = $SystemDB->getQueryResultSingleRow("SELECT id FROM scxnl_blacklist WHERE real_email_address = :email", ['email' => $email_address]);
        return $row ? $row['id'] : NULL;
    }       
    /**
     * Return all custom fields
     * @return array
     */
    public function getAllCustomFieldNames($visible_only = false)
    {
        $custom_field_app = new \App\Newsletters_CustomFields();
        $criteria = $visible_only ? 'status > 0' : '';
        $field_names = $custom_field_app->getAllItems('*', $criteria);
        return $field_names;
    }
    
    public function getTotalItemSearchCountWithCustomFields($where_condition)
    {
        global $SystemDB;
        
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields([], '');
        
        $where = $where_condition ? " WHERE {$where_condition} " : '';
        $sql2 = "SELECT COUNT(*) AS total_count from ($sql) tbl1 {$where}";
        $data = $SystemDB->getQueryResultSingleRow($sql2);
        
        /*
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields([], $where_condition);
        $sql2 = "SELECT COUNT(*) AS total_count from ($sql) tbl1 ";
        $data = $SystemDB->getQueryResultSingleRow($sql2);*/
        return $data['total_count'];
    }
    /**
     * Generate SQL query for select subscriber
     * @param array $field_names
     * @param string $where_condition
     * @param string $order_by
     * @param int $limit
     * @param int $offset
     * @return string
     */
    public function generateGetSubscriberSQLWithAllCustomFields ($field_names = [], $where_condition = '', $order_by = '', $limit = -1, $offset = -1)
    {
        $all_custom_fields = $this->getAllCustomFieldNames();
        $stmt = [];
        
        if ($all_custom_fields)
        {
            foreach ($all_custom_fields as $cf)
            {
                $st = "MAX( IF( fieldname_id = {$cf['id']}, field_value, NULL) ) AS {$cf['virtual_filename']}";
                if (!empty($field_names))
                {
                    if (in_array($cf['virtual_filename'], $field_names))
                    {
                        $stmt[] = $st;         
                    }
                } else
                {
                    $stmt[] = $st;
                }

            }
        }
        if ($stmt)            
            $join_stmt = ', '.implode(', ', $stmt);        
        else
            $join_stmt = '';
        
        $where = $where_condition ? " WHERE {$where_condition} " : '';
        $order = $order_by ? " ORDER BY {$order_by} " : '';
        $thelimit = $limit > -1 ? " LIMIT {$limit} " : '';
        $theoffset = $offset > -1 ? " OFFSET {$offset} " : '';
        $sql = "SELECT scxnl_subscribers.*, subscriber_id {$join_stmt} FROM scxnl_subscribers LEFT JOIN scxnl_subscriber_attributes ON scxnl_subscribers.id = scxnl_subscriber_attributes.subscriber_id  {$where} GROUP BY scxnl_subscribers.id {$order} {$thelimit} {$theoffset}";
        return $sql;
    }
    
    /**
     * Get SQL query result from subscribers
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param array $field_names
     * @param string $where_condition
     * @param string $order_by
     * @param int $limit
     * @param int $offset
     * @return array
     */
    public function searchAllSubscribersWithCustomFields($field_names = [], $where_condition = '', $order_by = '', $limit = 0, $offset = 0)
    {
        global $SystemDB;
        
        $where = $where_condition ? " WHERE {$where_condition} " : '';
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields($field_names, '', $order_by, -1, -1);
        $thelimit = $limit > -1 ? " LIMIT {$limit} " : '';
        $theoffset = $offset > -1 ? " OFFSET {$offset} " : '';
        
        $sql2 = "SELECT * from ($sql) tbl1 {$where} {$thelimit} {$theoffset}";
        //echo $sql2;die;
        return $SystemDB->getQueryResultArray($sql2);        
    }
    
    
    /**
     * Get SQL query result from subscribers
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param array $field_names
     * @param string $where_condition
     * @param string $order_by
     * @param int $limit
     * @param int $offset
     * @return array
     */
    public function getAllSubscribersWithCustomFields($field_names = [], $where_condition = '', $order_by = '', $limit = 0, $offset = 0)
    {
        global $SystemDB;
        
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields($field_names, $where_condition, $order_by, $limit, $offset);
        return $SystemDB->getQueryResultArray($sql);        
    }
    
    /**
     * Get Subscriber by GUID
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $guid
     * @return array
     */
    public function getSubscriberByGUID($guid)
    {
        global $SystemDB;
        
        $guid = sanitize_string($guid);
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields(NULL, "guid = {$guid}");
        return $SystemDB->getQueryResultSingleRow($sql);        
    }
    
    
    
    /**
     * Get Subscriber by ID
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $id
     * @return array
     */
    public function getSubscriberByID($id)
    {
        global $SystemDB;
        
        $id = (int) $id;
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields(NULL, "subscriber_id = {$id}");
        return $SystemDB->getQueryResultSingleRow($sql);        
    }
    
    /**
     * Get Subscriber By email address
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $email
     * @return array
     */
    public function getSubscriberByEmail($email)
    {
        global $SystemDB;
        
        $id = (int) $id;
        $real_email_address = sanitize_string(\App\Newsletters::getRealEmailAddress($email));
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields(NULL, "real_email_address = {$real_email_address}");
        return $SystemDB->getQueryResultSingleRow($sql);        
    }
    /**
     * View Main Page
     * @param int $pg
     * @return boolean
     */
    //_______________________________________________________________________________________________________________//
    public function viewMainPage($pg = 1) {
        return false;
    }
       
    
    /**
     * Validates save item. If there's an error, it will return an array
     * with one or more error string, otherwise it will return a boolean true
     * @global \App\Users $CurrentUser
     * @param array $datavalues
     * @return bool|array String array
     */
    public function getValidationErrorListBeforeSaveItem($datavalues)
    {
        $parent_error_list = parent::getValidationErrorListBeforeSaveItem($datavalues);
        $init_data_dir_error = $this->initializeDataDirectories();
        if ($init_data_dir_error)
        {
            $error_list[] = ___('Error while trying to create data directories. Maybe you can check the permission on the /web folder');
        }
        
        return array_merge($parent_error_list, $error_list);
    }
    
    /**
     * Get Subscriber By email address. Only a few fields added. GUID and Hash for unsubscribe link
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param string $email
     * @return array
     */
    public function getLimitedSubscriberInfoByEmail($email)
    {
        global $SystemDB;
        
        $id = (int) $id;
        $real_email_address = sanitize_string(\App\Newsletters::getRealEmailAddress($email));
        $sql = $this->generateGetSubscriberSQLWithAllCustomFields(
                ['id','email_address','real_email_address','guid','hash'], 
                "real_email_address = {$real_email_address}");
        return $SystemDB->getQueryResultSingleRow($sql);        
    }
    
            
    /**
     * Override default modify item data before save
     * @param array $datavalues
     */

    protected function modifyDataValuesBeforeSaveItem($datavalues) {
        $datavalues = parent::modifyDataValuesBeforeSaveItem($datavalues);
        
        $datavalues['real_email_address'] = \App\Newsletters::getRealEmailAddress($datavalues['email_address']);
            
        $attached_file = $_FILES['extra_attachment'];
        if (is_uploaded_file($_FILES['extra_attachment']['tmp_name'])) {
            if ($attached_file['error'] == UPLOAD_ERR_OK) {
                $attachment_file_name = $_FILES['extra_attachment']['name'];
                $new_attachment_filename =  pathinfo($attachment_file_name, PATHINFO_FILENAME) . '.' . pathinfo($attachment_file_name, PATHINFO_EXTENSION);

                // name the image the same as the vfname
                $datavalues['extra_attachment'] = $new_attachment_filename;
                
                $the_file = $this->getDataFileFullPath('attachments', $datavalues['guid']);
                $result =  move_uploaded_file($attached_file['tmp_name'], $the_file);
                
                $datavalues['extra_attachment_hash'] = sha1_file($the_file);
            }
        }
        if ( (int) $datavalues['delete_extra_attachment'] == 1)
        {
            $datavalues['extra_attachment'] = $datavalues['extra_attachment_hash'] = '';
        }
        return $datavalues;
    }
     /**
     * Do something after save item
     * @param array $datavalues
     * @param array $original_datavalues
     * @param array $previous_item
     * @param array $retval
     */
    protected function onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval)
    {
        parent::onAfterSaveItem($datavalues, $original_datavalues, $previous_item, $retval);
        if ($retval['status'] == SAVE_OK)
        {
            
            $this->setSubscriberCustomFieldValues($retval['id'], $original_datavalues);
        }
    }
    
    /**
     * Update subscriber custom field value directly
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param int $subscriber_id
     * @param int $field_id
     * @param string|int $field_value
     * @return string
     */
    protected function setSubscriberCustomFieldValue($subscriber_id, $field_id, $field_value)
    {
        global $SystemDB;
        
        $log_result = '';
        $subscriber_id   = (int) $subscriber_id;
        if ($subscriber_id == 0)
        {
            die( 'Subscriber ID cannot be zero when setting the custom field value');
        }        
        $existing_row = $SystemDB->getQueryResultSingleRow( "SELECT * FROM scxnl_subscriber_attributes WHERE subscriber_id = :subscriber_id AND fieldname_id = :fieldname_id", ['subscriber_id' => $subscriber_id, 'fieldname_id' => $field_id]);
        if ($existing_row)
        {
            $SystemDB->query("UPDATE scxnl_subscriber_attributes SET field_value = :field_value WHERE subscriber_id = :subscriber_id AND fieldname_id = :fieldname_id", ['subscriber_id' => $subscriber_id, 'fieldname_id' => $field_id, 'field_value' => $field_value]);
            $log_result = "old value = '{$existing_row['field_value']}', new value = '{$field_value}'";
        } else
        {
            $datavalues = ['fieldname_id' => $field_id, 'field_value' => $field_value,'subscriber_id' => $subscriber_id];
            $SystemDB->simpleInsertInto('scxnl_subscriber_attributes', $datavalues);
            $log_result = "new value = '{$field_value}'";
        }
        return $log_result;
    }
    
    /**
     * Given an array, insert or update the data values of this subscriber
     * @global \SCHLIX\cmsDatabase $SystemDB
     * @param type $id
     * @param type $datavalues
     */
    public function setSubscriberCustomFieldValues ($subscriber_id, $datavalues)
    {
        global $SystemDB;
        
        $subscriber_id   = (int) $subscriber_id;
        if ($subscriber_id == 0)
        {
            die( 'Subscriber ID cannot be zero when setting the custom field value with array');
        }
        $existing_subscriber = $this->getItemByID($subscriber_id);
        if (($existing_subscriber))
        {            
            $all_log_result = [];
            $custom_fields = $this->getAllCustomFieldNames();
            foreach ($custom_fields as $cf)
            {
                $custom_field_name = $cf['virtual_filename'];            
                $custom_field_id = $cf['id'];
                if (array_key_exists($custom_field_name, $datavalues))
                {
                    $custom_field_value = $datavalues[$custom_field_name];
                    $log_result = $this->setSubscriberCustomFieldValue($subscriber_id, $custom_field_id, $custom_field_value);
                    $all_log_result []= "{ {$custom_field_name}: [{$log_result}] }";
                }
            }
            $str_all_log_result = implode(',', $all_log_result);
            $this->subscriberLog->recordSubscriberActivity($subscriber_id, "Set data for subscriber ID# {$subscriber_id} with email {$existing_subscriber['email_address']}: {$str_all_log_result}", 'subscriber-data', 'info');
        }
        else
        {
            $this->eventLog->recordGenericEvent("Unable to set user attributes for subscriber #{$subscriber_id} because it cannot be found", 'subscriber-data', 'error');
        }
    }
    

    /**
     * Run Command
     * @param array $command
     * @return boolean
     */
    public function Run($command) {
        //$this->insertTest();
        switch ($command['action']) {
            case 'main': $this->viewMainPage($command['pg']);
                return true;
                break;
            default: return parent::Run($command);
        }
    }



}
